#' Plot grid-constrained layout of network vertices.
#'
#' \code{grid.plot} Plotting function for network vertices laid out on
#' a grid.
#'
#' This function provides a convenient way to plot results of
#' \code{\link{layout.grid.centroid.attraction}}.
#'
#'
#' @param layout A numerical matrix (as returned by \code{\link{layout.grid.centroid.attraction}}
#' in which case the cells of the matrix
#' will consist of indices referring to the columns of the distance matrix
#' provided to \code{\link{layout.grid.centroid.attraction}}).
#' @param colour.matrix A matrix representing colours in any of the R formats.
#' This matrix must be the same size and dimensions as the \code{layout} matrix.
#' Each element describes a colour for the vertex referred to by the
#' corresponding element in \code{layout}.
#' See example for details on how to use this.
#' @return Invisibly returns a matrix of colours corresponding to the \code{colour.matrix}
#' matrix, if provided, or the default colouring otherwise. Default colours are
#' grey for vertices and white for empty cells (used to pad the matrix to a
#' square grid).
#'
#' @examples
#'
#' \dontrun{
#' require(GridGraph)
#' # Load example data and create distance matrix
#' data(hipExpr)
#' network = cor(hipExpr, method="pearson", use="pairwise.complete.obs")
#' distance.matrix = 1 - (network * (network >= 0))
#'
#' # Perform layout
#' threshold = 0.5
#' grid.layout = layout.grid.centroid.attraction(distance.matrix, threshold,
#' iterations = 20)
#'
#' # Select a vertex (one of the genes/columns from the example data) and
#' # find neighbours (other genes that correlate with this seed)
#' seed.vertex = match("ENSMUSG00000004891", rownames(distance.matrix))
#' threshold = 0.5
#' seed.neighbours = which(distance.matrix[seed.vertex, ] <= threshold)
#'
#' # Create a martix of colours, background = grey, missing values = white,
#' # seed vertex = red, neighbour vertices = blue
#' grid.plot.colours = matrix(rep("#E0E0E0", length(grid.layout)),
#' ncol=ncol(grid.layout))
#' grid.plot.colours[is.na(grid.layout)] = "#FFFFFF"
#' grid.plot.colours[match(seed.neighbours, grid.layout)] = "#006699"
#' grid.plot.colours[match(seed.vertex, grid.layout)] = "#E03000"
#'
#' # Plot the layout
#' grid.plot(grid.layout, grid.plot.colours)
#' }
#'
#' @importFrom graphics par plot rasterImage
#' @importFrom grDevices as.raster
#'
#' @export
grid.plot = function(layout, colour.matrix=NULL){
	if(length(colour.matrix)==0){
		colour.matrix = matrix(rep("#E0E0E0", length(layout)), ncol=ncol(layout))
		colour.matrix[is.na(layout)] = "#FFFFFF"
	}
	default.mar = par()$mar
	par(mar=c(0, 0, 0, 0))
	plot(0, xlim=c(0, 1), type="n", axes=F, xaxs="i", yaxs="i", ylim=c(0, 1), xlab="", ylab="")
  raster = as.raster(colour.matrix)
	rasterImage(raster, xleft=0, ybottom=0, xright=1, ytop=1, interpolate=F)
	par(mar = default.mar)
	invisible(raster)
}
