#' Grid-constrained layout of network vertices.
#'
#' \code{layout.grid.centroid.attraction} Layout of network vertices by 
#' iterative convergence to the centroid of all neighbours.
#'
#' This function places each vertex onto a grid such that the Euclidean distance 
#' between any two cells is optimally close to the weight of the edge connecting 
#' the corresponding vertices. 
#' In each iteration of the layout, the centroid of the cells corresponding to 
#' all neighbor vertices is calculated and the new position of the vertex is 
#' moved towards this centroid (by a fraction of the distance determined by the 
#' \code{increment} parameter.
#' Displaced cells are shuffled along a mapped direct path between target cell 
#' and source vertex.
#' After several iterations, cells with converge to a layout where more 
#' highly-connected vertices are placed in more closely adjacent cells and 
#' unconnected (orphan) nodes are displaced to the edges.
#' 
#'
#' @param distance.matrix A matrix of \emph{distances} representing an undirected 
#' weighted network. Distances are smaller for more correlated vertices and can 
#' be generated by subtracting the Pearson correlation (or equivalent) from 1 
#' and setting all values above 1 to equal 1. 
#' @param threshold A threshold below which vertices are considered to be 
#' connected by an edge.
#' @param row.size.factor A factor indicating by how many rows the grid should
#'  be padded (see the details for an explanation).
#' @param col.size.factor A factor indicating by how many columns the grid 
#' should be padded (see the details for an explanation).
#' @param increment A value between 0 and 1 determining what proportion of the 
#' displacement to the centroid the new cell is placed. Larger values will 
#' speed up, but potentially destabilise, convergence. Default is 0.5 (half the 
#' displacement).
#' @param extend An optional matrix representing the state of a layout (as 
#' returned by a previous iteration of \code{\link{layout.grid.centroid.attraction}}). 
#' If provided, this will be taken as the starting state of the current layout. 
#' In this way, additional iterations can be added to a layout or graphs can be 
#' made at different time points during layout to observe convergence as an animation.
#' The default (NULL) will start with a new layout. It is important that the
#' random seed is set beforehand using \code{\link[base]{set.seed}}.
#' @param iterations The number of iterations that should be performed.
#' @return A matrix of indices corresponding to the rows in 
#' \code{distance.matrix} and NAs for non-vertex cells. 
#'
#' @examples
#'
#' \dontrun{
#' require(GridGraph)
#' # Trivial example using the "mtcars" data set.
#' distance.matrix <- 1 - cor(mtcars)
#' distance.matrix[which(distance.matrix > 1)] <- 1
#' grid.layout <- layout.grid.centroid.attraction(distance.matrix, 0.8, row.size.factor=4, 
#'                                                col.size.factor=4, 
#'                                                iterations=100)
#' image(t(apply(grid.layout > 0, 2, rev))) # Visualise
#' }
#' 
#' # See grid.plot for a more involved example.
#' 
#' @export
layout.grid.centroid.attraction = function(distance.matrix, threshold, row.size.factor=1, col.size.factor=1, increment=0.5, extend=NULL, iterations=1){
  # Create neighbour list
  neighbours = list(length=nrow(distance.matrix))
  for(i in 1:nrow(distance.matrix)){
    v.neighbours = which(distance.matrix[i, ] <= threshold)
    v.neighbours = v.neighbours[-which(v.neighbours==i)]
    neighbours[[i]] = v.neighbours
  }
  
  # Create grid and populate
  grid = matrix(nrow=ceiling(sqrt(nrow(distance.matrix) * row.size.factor)), ncol=ceiling(sqrt(nrow(distance.matrix) * col.size.factor)))
  vertices = 1:nrow(distance.matrix)
  if(length(extend)>0){
    grid = extend # Continue iterating on an existing layout
  }else{
    grid[1:nrow(distance.matrix)] = vertices # Add vertices
    grid[1:length(grid)] = sample(grid) # Randomise layout
  }
  
  # LOOP
  for(i in 1:iterations){
    for(v in vertices){
      # Get grid indices
      v.index = which(grid==v)
      if(length(neighbours[[v]]) > 0){ # If no neighbours, move nothing
        neighbour.indices = match(neighbours[[v]], grid)
        # Calculate centroid
        neighbour.coordinates = matrix.map(neighbour.indices, dim(grid))
        centroid.coordinates = lapply(lapply(neighbour.coordinates, mean), round)
        v.coordinates = matrix.map(v.index, dim(grid))
        # Displaced centroid is the distance to the real centroid multiplied by the 'increment' parameter
        row.displacement = ceiling(increment * (centroid.coordinates$row - v.coordinates$row))
        col.displacement = ceiling(increment * (centroid.coordinates$col - v.coordinates$col))
        displacement.length = round((row.displacement ^ 2 + col.displacement ^ 2) ^ 0.5)
        # Calculate the coordinates of the diagonal path (some cells may be repeated - this is fixed by making unique in the next step)
        row.displacement.coordinates = round(seq(v.coordinates$row, (v.coordinates$row + row.displacement), length.out=displacement.length))
        #- tryCatch({}, error=function(e){ print(paste(i, v, displacement.length, e)) })
        col.displacement.coordinates = round(seq(v.coordinates$col, (v.coordinates$col + col.displacement), length.out=displacement.length))
        #- tryCatch({}, error=function(e){ print(paste(i, v, displacement.length, e)) })
        # Shuffle cells along diagonal path to v cell...
        displacement.indices = unique(matrix.unmap(row.displacement.coordinates, col.displacement.coordinates, dim(grid)))
        if(length(displacement.indices)>1){ # If no-one's moving, save some execution time
          grid[displacement.indices[-length(displacement.indices)]] = grid[displacement.indices[-1]]
          # ...and move saved v to displaced centroid position
          grid[displacement.indices[length(displacement.indices)]] = v
        }
        #- if(length(which(table(grid)>1))) print(paste(i, v, displacement.length, which(table(grid)>1)))
      }
    }
  }
  return(grid)
}

